﻿/* menu.js */

var MenuStateObserver =
{
	observe: function( subject, topic, data )
	{
		// get kernel state
		var kernelState = CometMarksKernel.getState();
		switch( topic )
		{
			case CometMarksKernelDefine.event_update :
			case CometMarksKernelDefine.event_sync :	
			case CometMarksKernelDefine.event_overwrite :	
				if ( kernelState.is_login )
					this.showLoginState();
				else
					this.showLogoutState();
				break;
				
			case CometMarksKernelDefine.event_logout :
				this.showLogoutState();
				break;
				
			case CometMarksNotesDefined.event_note_status:
				this.showNotesStatus( data );
				break;
				
			case CometMarksStatusBarDefined.event_statusbarmenu_changed :
				this.changeStatusbarMenu();
				break;
				
			default :
				this.showLogoutState();
				break;
		}
	},
	
	changeStatusbarMenu : function()
	{
		var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var prefs = prefService.getBranch( "cometmarks." );
		var hide_statusicon = prefs.getBoolPref( "hidestatusicon" );
		
		if ( !hide_statusicon )// set check
		{
			document.getElementById("cometmarks_showstatusicon_menuitem").setAttribute( "checked", "true" );
		}
		else // remove check
		{
			document.getElementById("cometmarks_showstatusicon_menuitem").removeAttribute( "checked");
		}
	},
	
	showNotesStatus : function( status )
	{
		var panel = document.getElementById( "cometmarks_note_panel" );
		if ( status == CometMarksNotesDefined.status_close )
		{
		try{
			if ( panel == null )
				return;
				
			if ( panel.state == "open" ) // if open, close first
			{	
				// save temp data
				CometMarksGlobal.setData( CometMarksGlobalDefine.current_note_guid, CometmarksNotesHandler._temp_notes_data[CometmarksNotesHandler._current_label_id].guid ); // save temp note guid
				CometMarksGlobal.setData( CometMarksGlobalDefine.current_note_content, document.getElementById("panel_note_text").value ); // save temp note content
				
				// hide panel
				panel.hidePopup();
			}
		
			var note_menu = document.getElementById("cometmarks_show_note_menuitem");
			note_menu.removeAttribute( "checked" );
		}catch(e){write_log (e);}
		}
	},
	
	showLoginState : function()// login
	{
		// set bookmarks title
		this.setBookmarksTitle( true );
		
		// note item
		document.getElementById("cometmarks_show_note_menuitem").setAttribute("disabled", "false");
		
		document.getElementById("cometmarks_logout_menuitem").setAttribute("disabled", "false");
	},
	
	showLogoutState : function()// logout
	{
		// set bookmarks title
		this.setBookmarksTitle( false );
		
		// note item
		document.getElementById("cometmarks_show_note_menuitem").setAttribute("disabled", "true");
		
		document.getElementById("cometmarks_logout_menuitem").setAttribute("disabled", "true");
	},
	
	setBookmarksTitle : function( isLogin )
	{
		// get local language
		var ps = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var prefs = ps.getBranch("general.");
		var locate_name = prefs.getCharPref("useragent.locale");
		if ( locate_name == "en-US" || locate_name == "zh-CN" )
		{
			// set bookmarks title
			var bubble = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService).createBundle("chrome://cometmarks/locale/cometmarks.properties");
			var menu_obj = document.getElementById("bookmarksMenu");
			if ( !menu_obj )
				return;
				
			if ( isLogin )
				menu_obj.setAttribute( "label", bubble.GetStringFromName("cometmarks.online_login.book") );
			else
				menu_obj.setAttribute( "label", bubble.GetStringFromName("cometmarks.online_logout.book") );
		}
	}
};

// set cometmarks_guid if the loading url is bookmark
var urlBarListener = 
{
	QueryInterface: function(aIID)
	{
		if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
		   aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
		   aIID.equals(Components.interfaces.nsISupports))
			return this;
		throw Components.results.NS_NOINTERFACE;
	},
	
	onLocationChange: function(aProgress, aRequest, aURI)
	{
		var bookmarksService = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"].getService(Components.interfaces.nsINavBookmarksService);
		if ( !bookmarksService.isBookmarked(aURI) )
			return; 
			
		var BookmarkId_array = bookmarksService.getBookmarkIdsForURI(aURI, {});			
		var cometmarks_guid = bookmarksService.getItemGUID(BookmarkId_array[0]); // first one
		
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
		var gBrowser = wm.getMostRecentWindow("navigator:browser").getBrowser();
		var currentTab = gBrowser.getBrowserAtIndex( gBrowser.mTabContainer.selectedIndex ); 					 	
		currentTab.setAttribute( "cometmarks_guid", cometmarks_guid );// set guid for current tab
	},
	
	onStateChange: function() {},
	onProgressChange: function() {},
	onStatusChange: function() {},
	onSecurityChange: function() {},
	onLinkIconAvailable: function() {}
};

/* deal with menu xul */
var CometmarksMenuHandler = 
{
	init : function()
	{
	try{
		netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");	
	
		// register observer
		var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
		observerService.addObserver( MenuStateObserver, CometMarksKernelDefine.event_update, false );
		observerService.addObserver( MenuStateObserver, CometMarksKernelDefine.event_sync, false );
		observerService.addObserver( MenuStateObserver, CometMarksKernelDefine.event_overwrite, false );
		observerService.addObserver( MenuStateObserver, CometMarksKernelDefine.event_logout, false );
		observerService.addObserver( MenuStateObserver, CometMarksNotesDefined.event_note_status, false );
		observerService.addObserver( MenuStateObserver, CometMarksStatusBarDefined.event_statusbarmenu_changed, false );
		
		// to listen urlbar
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
		var gBrowser = wm.getMostRecentWindow("navigator:browser").getBrowser();
		gBrowser.addProgressListener( urlBarListener, Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT );
		
		// update menu-self
		var kernelState = CometMarksKernel.getState();
		if ( kernelState.is_login )
			MenuStateObserver.showLoginState();
		else
			MenuStateObserver.showLogoutState();
		
		// to check if the application start
		if ( !CometMarksRoutine.isHasPrivilege() ) // do nothing
			return;
			
		///////////////////////////////////////////////////////////////////
		// remove log last time
		delete_log();

		// get kernel state
		var kernelState = CometMarksKernel.getState();
		
		var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var prefs = prefService.getBranch("cometmarks.");
		
		// old account exist in perf.js
		if ( CometMarksObolete.getUserinfo() )
		{
			CometMarksObolete.setUserinfotoCometPassport();//set the userinfo into comet passport
			CometMarksObolete.Remove();// remove old data 
			return;
		}
		
		// get username and password
		var aUsername = CometPassport.getUsername();
		var aPassword = CometPassport.getPassword();
		if ( prefs.getBoolPref("autologin") && aUsername != "" && aPassword != "" )
		{	
			write_log( "cometmarks login account: " + aUsername );
			setTimeout( function(){ CometMarksKernel.setUserName( aUsername, aPassword ); }, 100 );
		}
		else
		{
			if ( !kernelState.is_login )
			{
				// logout
				CometMarksKernel.logOut();
			}
		}
	
		// to check if the first time install
		var ps = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService); 
		var prefs = ps.getBranch("cometmarks.");
		if ( !prefs.getBoolPref("first_install") )
			return;
		else	
		{
			prefs.setBoolPref("first_install", false);
			setTimeout( function(){ window.open("chrome://cometmarks/content/cometmarks_welcome.xul", "Welcome to CometMarks", "chrome,titlebar,dialog=yes,minimizable =no,toolbar=yes,centerscreen");	}, 5000 );
		}
	}catch(e){write_log(e);}
	},
	
	release : function()
	{
		// remove it
		var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
		observerService.removeObserver( MenuStateObserver, CometMarksKernelDefine.event_update );
		observerService.removeObserver( MenuStateObserver, CometMarksKernelDefine.event_sync );
		observerService.removeObserver( MenuStateObserver, CometMarksKernelDefine.event_overwrite );
		observerService.removeObserver( MenuStateObserver, CometMarksKernelDefine.event_logout );
		observerService.removeObserver( MenuStateObserver, CometMarksNotesDefined.event_note_status );
		observerService.removeObserver( MenuStateObserver, CometMarksStatusBarDefined.event_statusbarmenu_changed );
	}
};

// add global event listerner
window.addEventListener("load", CometmarksMenuHandler.init, false);
window.addEventListener("unload", CometmarksMenuHandler.release, false);